###############################################################
#                                                             #
# Template code to compute stresses in a half plane subject   #
# to a distributed load on its surface.                       #
#                                                             #
# "Contact Mechanics and Elements of Tribology" (CMET) course #
#                                                             #
# V.A. Yastrebov, CNRS                                        #
# MINES Paris, PSL University, Centre des materiaux           #
# Feb 2021-Feb 2023                                           #
#                                                             #
# Licence: CC0                                                #
#                                                             #
###############################################################

import sys
import numpy as np
import scipy.integrate as integrate
import matplotlib.pyplot as plt
import matplotlib 
from matplotlib import cm

matplotlib.rcParams.update(matplotlib.rcParamsDefault)
matplotlib.rcParams['figure.figsize'] = 10,8
matplotlib.rcParams.update({'font.size': 14})
# plt.rc('text', usetex=True)
# plt.rc('font', family='serif')

# Function to plot stress fields
def PlotField(X: np.dtype(float),Y: np.dtype(float),Z: np.dtype(float),label: str,filename: str) -> None:
  fig,ax = plt.subplots()
  ax.set_aspect(1)
  plt.xlabel("x/a")
  plt.ylabel("y/a")

  lev = np.linspace(np.min(Z/p0),np.max(Z/p0),NumColors)

  cs = ax.contourf(X/a, Y/a, Z/p0, levels=lev, cmap=cm.rainbow)
  ax.contour(X/a,Y/a,Z/p0,levels=lev,colors="k")
  cbar = fig.colorbar(cs)
  cbar.ax.set_ylabel(label)
  fig.savefig(filename)
  plt.show()

# Contact pressure is distributed from -a to a
a = 1.
# Poisson's ratio
v = 0.3
# Pressure constant
p0 = 10.

### HELPER on Python syntaxis ###
# product a*b
# power   a**b
# pi      np.pi
# square root np.sqrt(x)
#################################

# Pressure distribution
def pres(x):
  if abs(x) <= a:
    return p0 * np.sqrt(1-(x/a)**2) 
  else:
    return 0.

# Integrands
def fxx(x,y,s):
  return - 2 * pres(s)/np.pi * y *(x-s)**2 / ((x-s)**2 + y**2)**2
def fyy(x,y,s):
  return - 2 * pres(s)/np.pi * y**3 / ((x-s)**2 + y**2)**2
def fxy(x,y,s):
  return - 2 * pres(s)/np.pi * y**2 *(x-s) / ((x-s)**2 + y**2)**2

# Read arguments.
# Warning: if you do not know how to provide your code with argument or simply to accelerate it,
# you can comment the lines below until "# EO COMMENT" and uncomment two lines below "L=4*a,N=10"
if len(sys.argv) != 3:
  print("I need 2 arguments: 1 - box size (double), 2 - number of output points per side (int).")
  exit(1)
# Box size
L = float(sys.argv[1])
# Number of output points per side
N = int(sys.argv[2])
# EO COMMENT
# L = 4*a
# N = 10

# Stress arrays
sig_xx = np.zeros((N,N))
sig_yy = np.zeros((N,N))
sig_xy = np.zeros((N,N))
sig_zz = np.zeros((N,N))
sig_vm = np.zeros((N,N))

# Discretization
dl = L/float(N)

# Evaluate integrals on the grid x in (-L/2,L/2) and y in (-L,0), fill numpy arrays/matrices and save all data in a file
output = open("stress.data","w")
output.write("# [1] x [2] y [3] sig_xx [4] sig_yy [5] sig_xy\n")
# Loop over y points
y= -L+dl/2.
for i in range(N):
  x=-L/2.+dl/2
# Loop over x points
  for j in range(N): 
# Evaluate integrals
    sig_xx[i][j] = integrate.quad(lambda s: fxx(x,y,s), -a, a)[0]
    sig_yy[i][j] = integrate.quad(lambda s: fyy(x,y,s), -a, a)[0]
    sig_xy[i][j] = integrate.quad(lambda s: fxy(x,y,s), -a, a)[0]
    sig_zz[i][j] = v*(sig_xx[i][j]+sig_yy[i][j])
    output.write(str(x)+" "+str(y)+" "+str(sig_xx[i][j])+" "+str(sig_yy[i][j])+" "+str(sig_xy[i][j])+" "+str(sig_zz[i][j])+" "+str(sig_vm[i][j])+"\n")       
    x+=dl
  output.write("\n")
  y+=dl
output.close()    

# Create mesh grid (X,Y)
x = np.linspace(-L/2. + dl/2., L/2. - dl/2., N)
y = np.linspace(-L + dl/2., -dl/2., N)
X, Y = np.meshgrid(x, y)
# Number of isolines
NumColors = 11

# Plot results
PlotField(X,Y,sig_xx,"sigma_xx/p0","sigma_xx.pdf")
PlotField(X,Y,sig_yy,"sigma_yy/p0","sigma_yy.pdf")
PlotField(X,Y,sig_xy,"sigma_xy/p0","sigma_xy.pdf")
PlotField(X,Y,sig_zz,"sigma_zz/p0","sigma_zz.pdf")